///////////////////////////////////////////////////////////////////////////
////                                                                   ////
////                          XMLEngine.h                              ////
////                                                                   ////
//// Routines for reading and sending XML files.  These routines are   ////
//// designed to operate with minimal resources, for example:          ////
//// the parse routine doesn't need the whole XML file in memory at    ////
//// once, it can operate on the file at chunks at a time.             ////
////                                                                   ////
//// Note: This was developed using CCS V3.xxx, which had some ANSI    ////
//// defeciencies.  One major limitation was pointers to constants.    ////
//// So some of the code represnted in the XML Engine is a little      ////
//// ugly to work around these defecencies.                            ////
////                                                                   ////
//// The XML Engine consists of two parts: Reading a document and      ////
//// sending a document.  The read document routine will create the    ////
//// response information that will be needed by the send document     ////
//// routine.                                                          ////
////                                                                   ////
//// XMLFindMatchingEscapeElement(*strElement, *iParent,               ////
////  *iEscapeCode, *iRow) - Given the current strElement name (it is  ////
////     case sensitive), returns TRUE if it is found in the XML       ////
////     Database and iRow and iEscape code will be updated (iRow is   ////
////     the row in the XML database, iEscape is the escape code       ////
////     associated with this element).  If iParent is 0xFF, only      ////
////     checks root elements, if iParent is 0xFE it will ignore the   ////
////     parent field, else it will only return TRUE if a strElement   ////
////     is found in the database with the same parent as iParent.     ////
////                                                                   ////
///////////////////////////////////////////////////////////////////////////
////                                                                   ////
//// VERSION HISTORY:                                                  ////
////                                                                   ////
//// Author Date        Comments                                       ////
//// -----  -------     ----------------------------------             ////
//// DSR    Aug-21-2006 Initial Release                                ////
////                                                                   ////
///////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996, 2006 Custom Computer Services          ////
//// This source code may only be used by licensed users of the CCS C  ////
//// compiler.  This source code may only be distributed to other      ////
//// licensed users of the CCS C compiler.  No other use, reproduction ////
//// or distribution is permitted without written permission.          ////
//// Derivative programs created using this software in object code    ////
//// form are not restricted in any way.                               ////
///////////////////////////////////////////////////////////////////////////

#ifndef __XML_ENGINE__
#define __XML_ENGINE__

/*
#ifndef XML_ENGINE_MAX_ELEMENT_NAME_SIZE
#define XML_ENGINE_MAX_ELEMENT_NAME_SIZE 20
#endif

#ifndef XML_ENGINE_MAX_ELEMENT_VALUE_SIZE
#define XML_ENGINE_MAX_ELEMENT_VALUE_SIZE 80
#endif
*/

#define XML_ELEMENTS_COUNT (sizeof(gc_XMLEngineElements)/sizeof(XML_ENGINE_ELEMENTS))

#ifndef __XML_ENGINE_ELEMENTS__
#define __XML_ENGINE_ELEMENTS__
typedef struct
{
   char elementName[XML_PARSE_MAX_ELEMENT_NAME_SIZE];
   XML_ESCAPE_CODE parent;          //set to -1 if root element (no parent)
   int isParent:1;     //set to TRUE if this is a parent (has no values, only children)
   int isHidden:1;        //if TRUE, hide this element when you read the entire parent
   int isDisabled:1;      //if TRUE, this element is disabled (you can't use read or write it).
                        //this is very similar to isHidden, provided so you can easily
                        //alter your XML_ENGINE_ELEMENTS array to conditionally compile
                        //specific tags.
} XML_ENGINE_ELEMENTS;
#endif

typedef enum
{
   XML_ENGINE_EC_OK = 0,
   XML_ENGINE_EC_CONTINUE = 1,
   XML_ENGINE_EC_SYNTAX_ERROR = 2,
   XML_ENGINE_EC_UNKOWN_PARENT = 3,
   XML_ENGINE_EC_UNKOWN_ELEMENT = 4,
  //the following are returned by XMLEngineWriteEscape():
   XML_ENGINE_EC_BAD_DATA_RO = 5,
   XML_ENGINE_EC_BAD_DATA_RANGE = 6,
   XML_ENGINE_EC_BAD_DATA_TYPE = 7,
   XML_ENGINE_EC_BAD_DATA_ERROR = 8
} XML_ENGINE_EC;

///////////////////////////////////////////////////////////////////////////////
///
/// XMLGetDocumentStart(responses, responseMaxCount)
///
/// Tells the XML engine which elements to write back.
///
/// Inputs:
///   responses - an array of requested elements
///
///   responseMaxCount - maximum number of requested elements
///
///////////////////////////////////////////////////////////////////////////////
void XMLGetDocumentStart(XML_ESCAPE_CODE *responses, int responseMaxCount);

///////////////////////////////////////////////////////////////////////////////
///
/// XMLGetDocument(in)
///
/// Gets the XML document and places it to the specified location.
///
/// Inputs:
///   in - the location for the XML document to be compiled.
///
/// Outputs:
///   An XML engine error code. If XML_ENGINE_EC_CONTINUE, the function must
///   be called again.
///
///////////////////////////////////////////////////////////////////////////////
XML_ENGINE_EC XMLGetDocument(char *in);


///////////////////////////////////////////////////////////////////////////////
///
/// XMLSendDocumentStart(elements, elementsCount)
///
/// Gives the start of the send document function and the total number of
/// elements to be read.
///
/// Inputs:
///   elements - An array of the requested elements to be returned
///
///   elementsCount - the number of elements requested
///
///////////////////////////////////////////////////////////////////////////////
void XMLSendDocumentStart(XML_ESCAPE_CODE *elements, int elementsCount);


///////////////////////////////////////////////////////////////////////////////
///
/// XMLSendDocument(result, maxSize)
///
/// Sends the XML file to result up to maxSize
///
/// Inputs:
///   result - The location to which the XML document is saved
///
///   maxSize - Total number of chars allowed to save to ret (including
///            null termination)
///
/// Outputs:
///   only returns XML_ENGINE_EC_CONTINUE or XML_ENGINE_EC_OK
///
///////////////////////////////////////////////////////////////////////////////
XML_ENGINE_EC XMLSendDocument(char *result, int16 maxSize);

int XMLGetDocumentResponses(void);

int8 XMLFindMatchingEscapeElement(char *elementName, XML_ESCAPE_CODE currParentRow,
XML_ESCAPE_CODE *retEscapeCode, XML_ESCAPE_CODE *retFoundRow);


//Given tag PTR, finds escapecode (row) that this XML element can be 
//found in the table.  Returns TRUE if found (and ret is updated), FALSE
//if not.
int XMLEngineFindEscapeCode(char *ptr, XML_ESCAPE_CODE *ret, int1 showHidden);

//Returns TRUE if the specified element is a parent
int1 XMLIsParent(XML_ESCAPE_CODE which);

//If parent is -1, continues from last Get().  If parent isn't -1, starts
//a new search for childred of the specified parent.  Returns the next child
//of this parent.  Returns -1 if there is no children (remaining).
XML_ESCAPE_CODE XMLGetNextChild(XML_ESCAPE_CODE parent, XML_ESCAPE_CODE lastPosition=-1);

///////////////////////////////////////////////////////////////////////////////
///
/// XMLEngineReadEscape(escapeCode, ret, maxSize)
///
/// This is a callback function that must be provided by your application
///
/// Given the escape code, read the value and save it (up to maxSize) to
/// ret.
///
/// Inputs:
///   escapeCode - The read method.  Your call back
///         should know how to deal with each escapeCode that you use in your
///         gc_XMLEngineElements element structure.  This is of data type
///         XML_ESCAPE_CODE, which needs to be typedef'd in your application.
///         For small XML structures an int8 will be fine, but for larger
///         ones you will need an int16.  XML_ESCAPE_CODE could also be
///         an enum.
///
///   ret - Where to save the result
///
///   maxSize - Total number of chars allowed to save to ret (including
///            null termination)
///
/// Outputs:
///   The value being returned if it is an integer.  If it is not an integer,
///   then this will be undefined.
///
///////////////////////////////////////////////////////////////////////////////
int32 XMLEngineReadEscape(XML_ESCAPE_CODE escapeCode, char *ret, int16 maxSize);

///////////////////////////////////////////////////////////////////////////////
///
/// XMLEngineWriteEscape(escapeCode, str)
///
/// This is a callback function that must be provided by your application
///
/// Given the value stored in str, save it to the unit's memory using the
/// method provided by it's escapeCode.
///
/// Inputs:
///   escapeCode - The method to write the data to memory.  Your call back
///         should know how to deal with each escapeCode that you use in your
///         gc_XMLEngineElements element structure.
///
///   str - The incoming value to save
///
/// Outputs:
///   A return code:
///      XML_ENGINE_EC_OK - Data saved successfully
///      XML_ENGINE_EC_BAD_DATA_RO - Attempt to write to a read-only element
///      XML_ENGINE_EC_BAD_DATA_RANGE - Attempt to write a value that is out
///            of range (assigning a value of 2 where only 0 or 1 is valid)
///      XML_ENGINE_EC_BAD_DATA_TYPE - Attempt to write a value that is the
///            wrong type (assigning a string where only numbers are valid, etc)
///      XML_ENGINE_EC_BAD_DATA_ERROR - A general error has happened trying to
///            save the value (maybe the EEPROM it's writing to has failed)
///
///////////////////////////////////////////////////////////////////////////////
XML_ENGINE_EC XMLEngineWriteEscape(XML_ESCAPE_CODE escapeCode, char *str);

#endif
